// ==============================================================
// Ananke vessel class - a LEO momentum exchange tether
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#include "Ananke.h"


// ==============================================================
// Member initialization
// ==============================================================
const double Ananke::facilityFuel = 20000.0;


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void Ananke::clbkSetClassCaps (FILEHANDLE cfg)
{
	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource(facilityFuel);
	SetDefaultPropellantResource(hpr);

	// initialise meshes, beacons and light globes
	meshMgr.init();

	// Docking ports
	CreateDock(_V(0.0, 0.0, -20015.177)+meshMgr.meshes.at((UINT)FACILITY)->pos, _V(0,1,0), _V(1,0,0)); 	// Port 0 - at climber facility, old cable attach point for cableA
	CreateDock(_V(4.05, -0.565, 19996.0)+meshMgr.meshes.at((UINT)FACILITY)->pos, _V(1,0,0), _V(0,1,0));	// Port 1 - at counterweight, for supply/maintenance access
	CreateDock(_V(0.0, 0.0, -20001.05)+meshMgr.meshes.at((UINT)CABLE_A)->pos, _V(0,-1,0), _V(-1,0,0));	// Port 2 - at old cable attach point for cableB
	CreateDock(_V(0.0, 0.0, -20001.05)+meshMgr.meshes.at((UINT)CABLE_B)->pos, _V(0,-1,0), _V(-1,0,0));	// Port 3 - at old attach point for catcher
	CreateDock(_V(0.0, 0.0, -73.65)+meshMgr.meshes.at(CATCHER)->pos, _V(0,0,-1), _V(0,-1,0));			// Port 4 - payload attach point
	catcherDockIndex = 4;
}

// --------------------------------------------------------------
// Process buffered key events and KeyComm events
// --------------------------------------------------------------
int Ananke::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events
	if (Playback()) return 0; // don't allow manual user input during a playback

	// Process buffered key events here
	if (key==OAPI_KEY_L)
	{
		if (beaconState)
			beaconState = OFF;
		else
			beaconState = ON;

		meshMgr.switchBeacons(beaconState);
	}

	return 0;
}


// --------------------------------------------------------------
// Pre-step
// --------------------------------------------------------------
void Ananke::clbkPreStep (double SimT, double SimDT, double mjd)
{
	VECTOR3 dir0;

	meshMgr.preStep();

	GetCameraDefaultDirection(dir0);
}

// --------------------------------------------------------------
// Post creation
// --------------------------------------------------------------
void Ananke::clbkPostCreation()
{
}

// --------------------------------------------------------------
// Visual created
// --------------------------------------------------------------
void Ananke::clbkVisualCreated(VISHANDLE vis, int refcount)
{
	meshMgr.visualCreated(vis);
}

// --------------------------------------------------------------
// Visual destroyed
// --------------------------------------------------------------
void Ananke::clbkVisualDestroyed(VISHANDLE vis, int refcount)
{
	meshMgr.visualDestroyed(vis);
}

// --------------------------------------------------------------
// Generic cockpit loading
// --------------------------------------------------------------
bool Ananke::clbkLoadGenericCockpit()
{
	SetCameraOffset(_V(0,0,0));
	SetCameraDefaultDirection(_V(0,0,-1));
	SetCameraRotationRange(PI,PI,PI/2,PI/2);

	return true;
}

// --------------------------------------------------------------
// Virtual cockpit loading
// --------------------------------------------------------------
bool Ananke::clbkLoadVC(int id)
{
	return meshMgr.loadVC((Vc_Pos)id);
}

// ==============================================================
// Custom member functions
// ==============================================================

// --------------------------------------------------------------
// Returns angle between an attached payload's velocity vector
// and the superstructure's velocity vector. Assumes that the
// tether is rotating prograde in the orbital plane.
// --------------------------------------------------------------
double Ananke::getPayloadVVAngle()
{
	VESSELSTATUS2 VesselStatus;
	VECTOR3 VesselPos, VesselRef, RVelRef, RPosRef, H, dockPos, dockDir, dockRot;
	double c;

	VesselStatus.version = 2;
	GetStatusEx (&VesselStatus);

	GetDockParams(GetDockHandle(catcherDockIndex), dockPos, dockDir, dockRot);
	Local2Global(dockPos, VesselRef);
	GetSuperstructureCG(VesselPos);
	Local2Global(VesselPos, VesselPos);
	VesselRef = Normalise(VesselRef-VesselPos);

	RVelRef = Normalise(VesselStatus.rvel);
	RPosRef = Normalise(VesselStatus.rpos);

	H = Normalise(CrossProduct(Normalise(CrossProduct(RVelRef, RPosRef)),RVelRef)); // H gives the normal to a plane perpendicular to the orbit plane and coplanar with the velocity vector
	c = acos(DotProduct(VesselRef,H));
	if (acos(DotProduct(VesselRef,RVelRef)) < PI/2)
		c = -c;
	
	return c;
}

// --------------------------------------------------------------
// Returns true if payload attached
// --------------------------------------------------------------
bool Ananke::hasPayload()
{
	if (DockingStatus(catcherDockIndex))
		return true;
	else
		return false;
}

// --------------------------------------------------------------
// Releases an attached payload. Returns true if successful
// --------------------------------------------------------------
bool Ananke::releasePayload()
{
	return Undock(catcherDockIndex);
}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new Ananke (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (Ananke*)vessel;
}
