// ==============================================================
// Ananke Mesh Manager class - manages the meshes for the
//                             Ananke class vessel
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#ifndef ANANKE_MESH_MANAGER_H
#define ANANKE_MESH_MANAGER_H

#define STRICT
#define _CRT_SECURE_NO_DEPRECATE

#include "math.h"
#include "orbitersdk.h"
#include "ZTCMaths.h"
#include "AnankeTypeDefs.h"
#include "AnankeBeaconManager.h"
#include "AnankeLightGlobeManager.h"
#include <vector>
#include <string>

// ==============================================================
// Type definitions
// ==============================================================

// --------------------------------------------------------------
// MassDistributionType - identifies the mass distribution for
//                        PMI calcs
// --------------------------------------------------------------
enum MassDistributionType {PNT, // mass concentrated at one point
						   ROD // mass evenly distributed along an infinitely thin rod
						  };

// --------------------------------------------------------------
// MassObject class - holds information about masses that are
//					  associated with the mesh
//
// Vectors in this class are in mesh coordinates, not local
// vessel coordinates
// --------------------------------------------------------------
class MassObject {
public:
	double mass;
	MassDistributionType massDistribution;
	VECTOR3 pos; // position of the centre of mass of the MassObject
	VECTOR3 radius; // defined as:
					// - for massDistribution==ROD, the position of one end
					// - for massDistribution==POINT, value is ignored but should be set to _V(0,0,0) for safety
};

// --------------------------------------------------------------
// Mesh class - holds a single mesh and its associated info
// --------------------------------------------------------------
class Mesh {
public:
	VECTOR3 pos;
	double size;
	std::string fileName;
	BeaconManager beaconMgr;
	LightGlobeManager lightGlobeMgr;
	std::vector<MassObject*> masses;
	UINT meshIdx;
};

// --------------------------------------------------------------
// MeshManager class - manages all beacons
// --------------------------------------------------------------
class MeshManager {
public:
	MeshManager(OBJHANDLE hV);
	~MeshManager();

	void init();
	void preStep();
	void visualCreated(VISHANDLE vis);
	void visualDestroyed(VISHANDLE vis);
	void calcCog();
	void calcPmi();
	void shiftMeshes(VECTOR3 &shiftVector, bool shiftCG);
	void switchBeacons(LightState ls);
	bool loadVC(Vc_Pos vcPos);

	std::vector<Mesh*> meshes;

	VECTOR3 cog; // used for buffering init() cog offset

private:
	static const UINT numberOfMeshes = 4;
	static const double cableMassRate;
	OBJHANDLE hVessel;
	VESSEL2 *v;
};

#endif // ANANKE_MESH_MANAGER_H